/***********************************************************************************
    Copyright 2008 Charles F. Gammie and Xiaoyue Guan

                        HAM  version 1.0   (released Jan 15, 2008)

    This file is part of HAM. HAM is a program that solves non-relativistic
    hyperbolic partial differential equation in conservative form using
    high-resolution shock-capturing techniques. This version of HAM has been
    configured to solve the magnetohydrodynamic equations of motion in
    axisymmetry to evolve a shearing box model.

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of HAM:

    [1] Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, Supplement,
        174, 145


    The latest version of the HAM is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    HAM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    HAM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HAM; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

/*
 * Subroutine diag
 *
 * Performing diagnostics for the run
 * 
 */


#include "decs.h"


void
diag (call_code)
     int call_code;
{
  char dfnam[10];
  int i, j, k;
  double rho, v, u, ek, eth, e_fin, m_fin;
  double ebtot;
  FILE *dump_file, *eb_file;
  void dump ();
  double U[NP], p1, p2, p3, rmed, divb, divbmax;
  double ek1, ek2, ek3, eb1, eb2, eb3, etot, alpha;
  int imax, jmax;
  void primtoU (double *p, double *U);

  double X1, X2, test1, test2, test3;

  static int dump_cnt;
  static double e_init, eb_init, m_init, tdump, ebmax_time, ebmax;
  static FILE *ener_file;

  static double timage;
  char ifnam[50];
  static int im_cnt;
  FILE *im_file;

  ebmax = 0.0;
  ebtot = 0.0;
  ebmax_time = 0.;

  if (call_code == 0)
    {
      ener_file = fopen ("ener.out", "w");
      if (ener_file == NULL)
	{
	  fprintf (stderr, "error opening energy output file\n");
	  exit (1);
	}
      tdump = t + SMALL;
      timage = t + SMALL;
      dump_cnt = 0;
      im_cnt = 0;
    }

  /* calculate energies and alpha */
  bounds (p);
  p1 = p2 = p3 = rmed = divbmax = 0.;
  ek1 = ek2 = ek3 = 0.;
  eb1 = eb2 = eb3 = 0.;
  alpha = 0.;
  LOOP
  {
    X1 = (i + 0.5) * dx1 - 0.5 * L1;
    X2 = (j + 0.5) * dx2 - 0.5 * L2;
    primtoU (p[i][j], U);
    rmed += U[RHO] * dV;
    p1 += U[U1] * dV;
    p2 += U[U2] * dV;
    p3 += (U[U3] - q * W * X1 * U[RHO]) * dV;
  }
  LOOP
  {
    X1 = (i + 0.5) * dx1 - 0.5 * L1;
    X2 = (j + 0.5) * dx2 - 0.5 * L2;
    ek1 += 0.5 * p[i][j][U1] * p[i][j][U1] * p[i][j][RHO] * dV;

    ek2 += 0.5 * p[i][j][U2] * p[i][j][U2] * p[i][j][RHO] * dV;
    ek3 +=
      0.5 * (p[i][j][U3] - q * W * X1) * (p[i][j][U3] -
					  q * W * X1) * p[i][j][RHO] * dV;

    eb1 += 0.5 * p[i][j][B1] * p[i][j][B1] * dV;
    eb2 += 0.5 * p[i][j][B2] * p[i][j][B2] * dV;
    eb3 += 0.5 * p[i][j][B3] * p[i][j][B3] * dV;
    alpha += (p[i][j][RHO] * p[i][j][U1] * (p[i][j][U3] - q * W * X1)
	      - p[i][j][B1] * p[i][j][B3]) * dV;

    /* flux-ct defn */
    divb = fabs (0.5 * (p[i][j][B1] + p[i][j - 1][B1]
			- p[i - 1][j][B1] - p[i - 1][j - 1][B1]) / dx1 +
		 0.5 * (p[i][j][B2] + p[i - 1][j][B2]
			- p[i][j - 1][B2] - p[i - 1][j - 1][B2]) / dx2);

    if (divb > divbmax)
      {
	divbmax = divb;
	imax = i;
	jmax = j;
      }
  }

  fprintf (stderr, "divbmax: %g %d %d\n", divbmax, imax, jmax);

  etot = ek1 + ek2 + ek3 + eb1 + eb2 + eb3;
  ebtot = ek1 + eb2 + eb3;
  alpha /= rmed * csq;

  if (ebtot > ebmax)
    {
      ebmax = ebtot;
      ebmax_time = t;
    }
  /* initial state */
  if (call_code == 0)
    {
      e_init = etot;
      m_init = rmed;
      eb_init = ebtot;
    }
  /* final dianostic */
  else if (call_code == 2)
    {
      e_fin = etot;
      m_fin = rmed;
      fprintf (stderr, "\n\nEnergy: ini,fin,del: %g %g %g\n",
	       e_init, e_fin, (e_fin - e_init) / e_init);
      fprintf (stderr, "mass: ini,fin,del: %g %g %g\n",
	       m_init, m_fin, (m_fin - m_init) / m_init);
    }
  /* write to ener_file at the end of every time step */
  else
    {
      fprintf (ener_file,
	       "%10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g %10.5g\n",
	       t, etot, ek1, ek2, ek3, eb1, eb2, eb3, alpha, p1, p2, p3,
	       rmed);
      fflush (ener_file);
    }


  /* dump at regular intervals */

  if (t >= tdump || call_code == 2 || call_code == 0)
    {
      sprintf (dfnam, "dump%03d", dump_cnt);
      dump_file = fopen (dfnam, "w");

      if (dump_file == NULL)
	{
	  fprintf (stderr, "error opening dump file\n");
	  exit (2);
	}

      dump (dump_file);
      fclose (dump_file);

      dump_cnt++;
      tdump += DTd;
    }


  /* make image of density at regular intervals */

  if (t >= timage || call_code == 2)
    {
      sprintf (ifnam, "im%04d", im_cnt);
      im_file = fopen (ifnam, "w");

      if (im_file == NULL)
	{
	  fprintf (stderr, "error opening image file\n");
	  exit (2);
	}

      image (im_file);
      fclose (im_file);

      im_cnt++;
      timage += DTi;
    }

}
