/***********************************************************************************
    Copyright 2008 Charles F. Gammie and Xiaoyue Guan

                        HAM  version 1.0   (released Jan 15, 2008)

    This file is part of HAM. HAM is a program that solves non-relativistic
    hyperbolic partial differential equation in conservative form using
    high-resolution shock-capturing techniques. This version of HAM has been
    configured to solve the magnetohydrodynamic equations of motion in
    axisymmetry to evolve a shearing box model.

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of HAM:

    [1] Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, Supplement,
        174, 145


    The latest version of the HAM is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    HAM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    HAM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HAM; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/


/*
 * Subroutines Utoprim, primtoU, primto1flux, primto2flux
 * 
 *  More Physics 
 *
 */

#include "decs.h"

/* calculate primitive variables from the conserved quantities */
void
Utoprim (double *U, double *pr, int i, int j)
{
  double rho, p1, p2, p3, b1, b2, b3;
  double X1;

  rho = U[RHO];
  p1 = U[U1];
  p2 = U[U2];
  p3 = U[U3];
  b1 = U[B1];
  b2 = U[B2];
  b3 = U[B3];

  pr[RHO] = rho;
  pr[U1] = p1 / rho;
  pr[U2] = p2 / rho;
  pr[U3] = p3 / rho;
  pr[B1] = b1;
  pr[B2] = b2;
  pr[B3] = b3;

}

/* calculate fluxes in the x1-direction */
void
primto1flux (double *pr, double *f, int i, int j)
{
  double rho, v1, v2, v3, b1, b2, b3;
  double X1;

  rho = pr[RHO];
  v1 = pr[U1];
  v2 = pr[U2];
  v3 = pr[U3];
  b1 = pr[B1];
  b2 = pr[B2];
  b3 = pr[B3];

  f[RHO] = rho * v1;
  f[U1] =
    rho * v1 * v1 - b1 * b1 + rho * csq + 0.5 * (b1 * b1 + b2 * b2 + b3 * b3);
  f[U2] = rho * v1 * v2 - b1 * b2;
  f[U3] = rho * v1 * v3 - b1 * b3;
  f[B1] = 0.;
  f[B2] = b2 * v1 - b1 * v2;
  f[B3] = b3 * v1 - b1 * v3;
}


/* calculate fluxes in the x2-direction */
void
primto2flux (double *pr, double *f, int i, int j)
{
  double rho, v1, v2, v3, b1, b2, b3;
  double X1;

  rho = pr[RHO];
  v1 = pr[U1];
  v2 = pr[U2];
  v3 = pr[U3];
  b1 = pr[B1];
  b2 = pr[B2];
  b3 = pr[B3];

  f[RHO] = rho * v2;
  f[U1] = rho * v2 * v1 - b2 * b1;
  f[U2] =
    rho * v2 * v2 - b2 * b2 + rho * csq + 0.5 * (b1 * b1 + b2 * b2 + b3 * b3);
  f[U3] = rho * v2 * v3 - b2 * b3;
  f[B1] = b1 * v2 - b2 * v1;
  f[B2] = 0.;
  f[B3] = b3 * v2 - b2 * v3;

}

/* calculate conserved quantities from the primitive variables */

void
primtoU (double *pr, double *U, int i, int j)
{
  double rho, v1, v2, v3, b1, b2, b3;
  double X1;

  rho = pr[RHO];
  v1 = pr[U1];
  v2 = pr[U2];
  v3 = pr[U3];
  b1 = pr[B1];
  b2 = pr[B2];
  b3 = pr[B3];

  U[RHO] = rho;
  U[U1] = rho * v1;
  U[U2] = rho * v2;
  U[U3] = rho * v3;
  U[B1] = b1;
  U[B2] = b2;
  U[B3] = b3;

}
