/***********************************************************************************
    Copyright 2008 Charles F. Gammie and Xiaoyue Guan

                        HAM  version 1.0   (released Jan 15, 2008)

    This file is part of HAM. HAM is a program that solves non-relativistic
    hyperbolic partial differential equation in conservative form using
    high-resolution shock-capturing techniques. This version of HAM has been
    configured to solve the magnetohydrodynamic equations of motion in
    axisymmetry to evolve a shearing box model.

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of HAM:

    [1] Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, Supplement,
        174, 145


    The latest version of the HAM is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    HAM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    HAM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HAM; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

/*
 * Subroutine step_ch
 *
 * 
 * Handle the sequence of one time step
 *
 */

#include "decs.h"

/* HLL  method */
#define MAX(a,b) ((a) > (b) ? (a) : (b))

void
step_ch ()
{
  double timestep ();
  void advance (double (*p_i)[N2 + 2 * NG][NP],
		double (*p_h)[N2 + 2 * NG][NP], double Dt,
		double (*p_f)[N2 + 2 * NG][NP]);

  /* evaluate timestep */
  dt = timestep ();

  /* half-step */
  advance (p, p, 0.5 * dt, ph);

  bounds (ph);

  /* full-step */
  advance (p, ph, dt, p);

  bounds (p);

  /* increment time */
  t += dt;

}


/* 
 * Subroutine advance
 *
 * Update primitive variables in one time step
 *         -- flux calculation: Harten, Lax & van Leer (HLL) scheme 
 *         -- constrained transport
 *         -- update conserved variables from finite differencing
 *         -- evaluate primitive variables from conserved variables       
 */

void
advance (double (*p_i)[N2 + 2 * NG][NP],
	 double (*p_h)[N2 + 2 * NG][NP],
	 double Dt, double (*p_f)[N2 + 2 * NG][NP])
{
  int i, j, k;
  double cms;
  double dU[NP];
  double cmax1, cmax2, cmin1, cmin2;
  double cmax1l, cmax1r, cmax2l, cmax2r, cmin1l, cmin1r, cmin2r, cmin2l;
  static double Uh[NP], U[NP];
  static double pl1[NP], pr1[NP], Fl1[NP], Fr1[NP], Ul1[NP], Ur1[NP];
  static double pl2[NP], pr2[NP], Fl2[NP], Fr2[NP], Ul2[NP], Ur2[NP];
  void primtoU (double *pb, double *Ub, int i, int j);
  void Utoprim (double *Ua, double *pa, int i, int j);
  void primto1flux (double *pa, double *Fa, int i, int j);
  void primto2flux (double *pa, double *Fa, int i, int j);
  double cms_func (double *prim_vec);
  void source (int i, int j, double *prim_vec, double *dU);
  void flux_ct ();

  /* evaluate slopes of primitive variables */
  /* assumes BCs have been set */
  for (i = -1; i < N1 + 1; i++)
    for (j = -1; j < N2 + 1; j++)
      {
	PLOOP
	{
	  dq1[i][j][k] = slope_lim (p_h[i - 1][j][k],
				    p_h[i][j][k], p_h[i + 1][j][k]);
	}
      }
  for (i = -1; i < N1 + 1; i++)
    for (j = -1; j < N2 + 1; j++)
      {
	PLOOP
	{
	  dq2[i][j][k] = slope_lim (p_h[i][j - 1][k],
				    p_h[i][j][k], p_h[i][j + 1][k]);
	}
      }

  /* evaluate zone boundary fluxes at half-step */
  /* first x1 fluxes */
  for (i = 0; i < N1 + 1; i++)
    for (j = -1; j < N2 + 1; j++)
      {
	PLOOP
	{
	  pl1[k] = p_h[i - 1][j][k] + 0.5 * dq1[i - 1][j][k];
	  pr1[k] = p_h[i][j][k] - 0.5 * dq1[i][j][k];
	}

	primtoU (pl1, Ul1, i, j);
	primtoU (pr1, Ur1, i, j);

	primto1flux (pl1, Fl1, i, j);
	primto1flux (pr1, Fr1, i, j);

	cms = cms_func (pl1);
	cmax1l = pl1[U1] + cms;
	cmin1l = pl1[U1] - cms;
	cms = cms_func (pr1);
	cmax1r = pr1[U1] + cms;
	cmin1r = pr1[U1] - cms;

	cmax1 = fabs (MAX (MAX (0., cmax1l), cmax1r));
	cmin1 = fabs (MAX (MAX (0., -cmin1l), -cmin1r));

	PLOOP
	{
	  F1[i][j][k] = (cmax1 * Fl1[k] + cmin1 * Fr1[k]
			 - cmax1 * cmin1 * (Ur1[k] - Ul1[k])) / (cmax1 +
								 cmin1);
	}
      }

  /* then x2 fluxes */
  for (i = -1; i < N1 + 1; i++)
    for (j = 0; j < N2 + 1; j++)
      {
	PLOOP
	{
	  pl2[k] = p_h[i][j - 1][k] + 0.5 * dq2[i][j - 1][k];
	  pr2[k] = p_h[i][j][k] - 0.5 * dq2[i][j][k];
	}
	primtoU (pl2, Ul2, i, j);
	primtoU (pr2, Ur2, i, j);

	primto2flux (pl2, Fl2, i, j);
	primto2flux (pr2, Fr2, i, j);

	cms = cms_func (pl2);
	cmax2l = pl2[U2] + cms;
	cmin2l = pl2[U2] - cms;
	cms = cms_func (pr2);
	cmax2r = pr2[U2] + cms;
	cmin2r = pr2[U2] - cms;

	cmax2 = fabs (MAX (MAX (0., cmax2l), cmax2r));
	cmin2 = fabs (MAX (MAX (0., -cmin2l), -cmin2r));

	PLOOP
	{
	  F2[i][j][k] = (cmax2 * Fl2[k] + cmin2 * Fr2[k]
			 - cmax2 * cmin2 * (Ur2[k] - Ul2[k])) / (cmax2 +
								 cmin2);
	}
      }

  /* the constrained transport */
  flux_ct ();

  LOOP
  {
    /* calculate conserved quantities */
    primtoU (p_i[i][j], U, i, j);

    /* the source term */
    source (i, j, p_h[i][j], dU);

    /* evolve conserved quantities */
    PLOOP
    {
      U[k] += Dt * (-(F1[i + 1][j][k] - F1[i][j][k]) / dx1 -
		    (F2[i][j + 1][k] - F2[i][j][k]) / dx2 + dU[k]);
    }

    /* recover new primitive variables */
    Utoprim (U, p_f[i][j], i, j);
  }

  //fixup(p_f); 

  /* done! */
}

#undef MAX

/* this overestimates wavespeeds slightly.
	It's worst when beta = 1 */
double
cms_func (double *prim_vec)
{
  double va2, cms;

  va2 = (prim_vec[B1] * prim_vec[B1] +
	 prim_vec[B2] * prim_vec[B2] +
	 prim_vec[B3] * prim_vec[B3]) / prim_vec[RHO];

  cms = sqrt (csq + va2);


  return (cms);
}

double
timestep ()
{
  int i, j;
  double dtsqijmin, cms, c1, c2, dtsqij, dt, cms_func (double *prim_vec);

  dtsqijmin = 1.e20;
  LOOP
  {
    /* evaluate wave speeds */
    cms = cms_func (p[i][j]);
    c1 = fabs (p[i][j][U1]) + cms;
    c2 = fabs (p[i][j][U2]) + cms;

    dtsqij = 0.5 / (c1 * c1 / (dx1 * dx1) + c2 * c2 / (dx2 * dx2));
    if (dtsqij < dtsqijmin)
      dtsqijmin = dtsqij;
  }
  dt = cour * sqrt (dtsqijmin);

  /* don't step beyond end of run */
  if (t + dt > tf)
    dt = tf - t;

  return (dt);
}

/* handle source terms */
void
source (int i, int j, double *prim_vec, double *dU)
{
  double X1;
  int k;

  X1 = (i + 0.5) * dx1 - 0.5 * L1;

  PLOOP dU[k] = 0.;

#if SHBOX
  dU[U1] = -2. * W * prim_vec[RHO] * prim_vec[U3]
    + 2. * q * W * W * prim_vec[RHO] * X1;
  dU[U3] = 2. * W * prim_vec[RHO] * prim_vec[U1];
#endif

}
